#-------------------------- 'iocode' Module --------------------------
my $output_ja_code;
my $input_ja_code;
my $LANGUAGE;
my $default_input_ja_code = '';
my $last_input_ja_code = '';
my $doc_input_ja_code = '';

sub set_output_ja_code{
    my $cs = shift;
    if ($cs =~ /euc/i){
        $output_ja_code = 'EUC-JP';
    }elsif ($cs =~ /shift|sj/i){
        $output_ja_code = 'Shift_JIS';
    }else{
        $output_ja_code = 'ISO-2022-JP';
    }
}

sub get_output_ja_code{
    return $output_ja_code;
}

sub get_input_ja_code{
    return $input_ja_code;
}

sub get_last_input_ja_code{
    return $input_ja_code;
}

sub set_doc_input_ja_code{
    $doc_input_ja_code = shift;
}

sub set_default_input_ja_code{
    $default_input_ja_code = shift;
}

sub get_language{
    $LANGUAGE = 'en' if ($LANGUAGE !~ /^[A-Z][A-Z0-9_,+@\-\.=]*$/i);

    return $LANGUAGE;
}

sub lang_set{
    my $key = shift;
    return unless defined $key && length $key;

    if ($key =~ /ja|jp|jis|sj/i){
        $LANGUAGE = 'ja';
        &set_output_ja_code($key);
    }else{
        $LANGUAGE = 'en';
    }
    $key = shift if @_;

    if ($key =~ /ja|jp|jis|sj/i){
        $input_ja_code = ($key =~ /sj|shift/i)? 'Shift_JIS': 'EUC-JP';
    }
}

sub lang{
    my ($x, $y) = @_;
    $LANGUAGE = 'ja';
    $input_ja_code = 'EUC-JP';
    $output_ja_code = 'EUC-JP';
    $input_ja_code = $output_ja_code = 'Shift_JIS' if $^O =~ /win|os2|dos|hp/i;

    if (defined $ENV{'GATEWAY_INTERFACE'}){
        $output_ja_code = 'ISO-2022-JP';
        $input_ja_code = 'EUC-JP';
        if (defined $ENV{'HTTP_ACCEPT_LANGUAGE'}
            && length $ENV{'HTTP_ACCEPT_LANGUAGE'}
            && $ENV{'HTTP_ACCEPT_LANGUAGE'} !~ /^ja/i){
            $LANGUAGE = 'en';
        }
    }

    my $key;
    &lang_set($x) if defined $x && $x ne '';
    &lang_set($ENV{'LANG'});
    &lang_set($ENV{'LC_MESSAGES'}, $ENV{'LC_CTYPE'});
    &lang_set($ENV{'LC_ALL'});
    &lang_set($ENV{'LANGUAGE'});
    &lang_set($y) if defined $y && $y ne '';
    return $output_ja_code;
}

sub euc_to_euc{
    my $ptr = shift;
    my $str = (ref($ptr) eq 'SCALAR')? $$ptr: $ptr;
    my $para = ref($_[0]) eq 'HASH'? shift: '';
    my $euc = '';
    my $stat = 0x017;

    my $x0208alpha_to_ascii = 0;
    my $x0201kana_to_x0208 = 0;
    my $summary = 0;

    if (ref($para) eq 'HASH'){
        $x0208alpha_to_ascii = $para->{'a2a'} if defined $para->{'a2a'};
        $x0201kana_to_x0208 = $para->{'k2e'} if defined $para->{'k2e'};
        $summary = $para->{'summary'} if defined $para->{'summary'};
    }
    while (@_){
        my $tmp = shift;
        $x0208alpha_to_ascii = 1 if $tmp eq 'a2a';
        $x0201kana_to_x0208 = 1 if $tmp eq 'k2e';
        $summary = 1 if $tmp eq 'akk';
    }

    while (length $str){
        if ($str =~ s/^([\x00-\x7f]|(\x8e[\xa1-\xdf])+|[\xa1-\xfe][\xa1-\xfe])//){
            my $c1 = ord $1;
            if ($c1 < 0x80){
                $euc .= $1;
            }elsif ($c1 == 0x8e){
                my $str = $1;
                $str = &x0201kana_to_x0208($str) if $x0201kana_to_x0208;
                $euc .= $str;
                $stat &= ~1;
            }else{
                my $c = $1;
                if ($c =~ /^[\xa9-\xaf\xf5-\xfe]/){
                    $stat &= ~2;
                    $stat &= ~4 if $c =~ /^[\xaf\xf6\xf7\xfd-\xfe]/;
                }else{
                    $stat |= 8;
                    $c = &x0208alpha_to_ascii($c) if $x0208alpha_to_ascii;
                }
                $euc .= $c unless $summary
                    && $c =~ /^[\xa8-\xaf\xf5-\xfe]/;
            }
        }else{
            substr($str, 0, 1) = '';
            $stat = 0;
        }
    }
    $para->{'stat'} = $stat if ref($para) eq 'HASH';
    $$ptr = $euc, return $stat if ref($ptr) eq 'SCALAR';
    return $euc;
}

sub shiftjis_to_euc{
    my $ptr = shift;
    my $str = (ref($ptr) eq 'SCALAR')? $$ptr: $ptr;
    my $para = ref($_[0]) eq 'HASH'? shift: '';
    my $euc = '';
    my $stat = 0x017;

    my $x0208alpha_to_ascii = 0;
    my $x0201kana_to_x0208 = 0;
    my $summary = 0;

    if (ref($para) eq 'HASH'){
        $x0208alpha_to_ascii = $para->{'a2a'} if defined $para->{'a2a'};
        $x0201kana_to_x0208 = $para->{'k2e'} if defined $para->{'k2e'};
        $summary = $para->{'akk'} if defined $para->{'akk'};
    }
    while (@_){
        my $tmp = shift;
        $x0208alpha_to_ascii = 1 if $tmp eq 'a2a';
        $x0201kana_to_x0208 = 1 if $tmp eq 'k2e';
        $summary = 1 if $tmp eq 'akk';
    }

    while (length $str){
        if ($str =~ s/^([\x00-\x7f]+|[\xa1-\xdf]+|[\x81-\x9f\xe0-\xef][\x40-\x7e\x80-\xfc])//
            ){
            my $c1 = ord $1;
            if ($c1 < 0x80){
                $euc .= $1;
            }elsif (0xa1 <= $c1 && $c1 <= 0xdf){
                my $str = $1;
                $str =~ s/(.)/\x8e$1/g;
                $str = &x0201kana_to_x0208($str) if $x0201kana_to_x0208;
                $euc .= $str;
                $stat &= ~1;
            }else{
                my $c2 = ord substr($1, 1);
                $c1 += ($c1 - 0x60) & 0x7f;
                if ($c2 < 0x9f){
                    $c1--;
                    $c2 += ($c2 < 0x7f) + 0x60;
                }else{
                    $c2 += 2;
                }
                my $c = chr($c1) . chr($c2);
                if ($c =~ /^[\xa9-\xaf\xf5-\xfe]/){
                    $stat &= ~2;
                    $stat &= ~4 if $c =~ /^[\xaf\xf6\xf7\xfd-\xfe]/;
                }else{
                    $stat |= 8;
                    $c = &x0208alpha_to_ascii($c) if $x0208alpha_to_ascii;
                }
                $euc .= $c unless $summary
                    && $c =~ /^\xa1[^\xbc]|[\xa2\xa8-\xaf]/;
            }
        }else{
            $str =~ s/^.//;
            $stat = 0;
        }
    }
    $para->{'stat'} = $stat if ref($para) eq 'HASH';
    $$ptr = $euc, return $stat if ref($ptr) eq 'SCALAR';
    return $euc;
}

sub x0201kana_to_x0208{
    my $str = shift;
    $str =~ s/\x8e([\xb3\xb6-\xc4\xca-\xce]\x8e\xde|[\xca-\xce]\x8e\xdf|.)/{
        my $s = (3 == length $1)? (ord(substr($1, 2)) - 0xde + 1): 0;
        my $hi = ord $1;
        my $lo = (0xA3, 0xD6, 0xD7, 0xA2, 0xA6, 0xF2, 0xA1, 0xA3,
                  0xA5, 0xA7, 0xA9, 0xE3, 0xE5, 0xE7, 0xC3, 0xBC,
                  0xA2, 0xA4, 0xA6, 0xA8, 0xAA, 0xAB, 0xAD, 0xAF,
                  0xB1, 0xB3, 0xB5, 0xB7, 0xB9, 0xBB, 0xBD, 0xBF,
                  0xC1, 0xC4, 0xC6, 0xC8, 0xCA, 0xCB, 0xCC, 0xCD,
                  0xCE, 0xCF, 0xD2, 0xD5, 0xD8, 0xDB, 0xDE, 0xDF,
                  0xE0, 0xE1, 0xE2, 0xE4, 0xE6, 0xE8, 0xE9, 0xEA,
                  0xEB, 0xEC, 0xED, 0xEF, 0xF3, 0xAB, 0xAC
                  )[$hi - 0xa1];
        $hi = (0xa6 <= $hi && $hi != 0xb0 && $hi <= 0xdd)? 0xa5: 0xa1;
        $lo = ($hi == 0xb3 && $s)? 0xdd: ($lo + $s);
        chr($hi) . chr($lo);
    }/ge;
    $str;
}

sub x0208_to_x0201kana{
    my $str = shift;
    $str =~ s/([\x80-\xfe].|[\x00-\x7f]+)/{
        my $ret = $1;
        my $hi = ord $ret;
        my ($c, @tbl);
        if ($hi == 0xa1){
            @tbl = ("", "\xA4", "\xA1", "", "", "\xA5", "", 
                    "", "", "", "\xDE", "\xDF", "", "", "", 
                    "", "", "", "", "", "", "", "", 
                    "", "", "", "", "\xB0", "", "", "", 
                    "", "", "", "", "", "", "", "", 
                    "", "", "", "", "", "", "", "", 
                    "", "", "", "", "", "", "\xA2", "\xA3");
        }elsif ($hi == 0xa5){
            @tbl = ("\xA7", "\xB1", "\xA8", "\xB2", "\xA9", "\xB3", "\xAA", 
                    "\xB4", "\xAB", "\xB5", "\xB6", "\xB6\xDE", "\xB7", "\xB7\xDE", "\xB8", 
                    "\xB8\xDE", "\xB9", "\xB9\xDE", "\xBA", "\xBA\xDE", "\xBB", "\xBB\xDE", "\xBC", 
                    "\xBC\xDE", "\xBD", "\xBD\xDE", "\xBE", "\xBE\xDE", "\xBF", "\xBF\xDE", "\xC0", 
                    "\xC0\xDE", "\xC1", "\xC1\xDE", "\xAF", "\xC2", "\xC2\xDE", "\xC3", "\xC3\xDE", 
                    "\xC4", "\xC4\xDE", "\xC5", "\xC6", "\xC7", "\xC8", "\xC9", "\xCA", 
                    "\xCA\xDE", "\xCA\xDF", "\xCB", "\xCB\xDE", "\xCB\xDF", "\xCC", "\xCC\xDE", "\xCC\xDF", 
                    "\xCD", "\xCD\xDE", "\xCD\xDF", "\xCE", "\xCE\xDE", "\xCE\xDF", "\xCF", "\xD0", 
                    "\xD1", "\xD2", "\xD3", "\xAC", "\xD4", "\xAD", "\xD5", "\xAE", 
                    "\xD6", "\xD7", "\xD8", "\xD9", "\xDA", "\xDB", "", "\xDC", 
                    "", "", "\xA6", "\xDD", "\xB3\xDE");
        }
        if (defined($c = $tbl[ord(substr($ret, 1)) - 0xa1]) && $c){
            $c =~ s\/(.)\/\x8e$1\/g;
            $ret = $c;
        }
        $ret;
    }/ge;
    $str;
}

sub x0208alpha_to_ascii{
    my $str = shift;
    my $ret = '';
    while ($str =~ s/^([\x00-\x7f]|[\x8e\xa1-\xfe][\xa1-\xfe])//s){
        my $tmp = my $c = $1;
        if ($c =~ s/^([\xa1\xa3])//){
            if ($1 eq "\xa1"){
                $ret .= $c, next if $c =~ tr/\xa1\xa4\xa5\xa7\xa8\xa9\xaa\xae\xb0\xb2\xbf\xc3\xc7\xc9\xca\xcb\xce\xcf\xd0\xd1\xdc\xdd\xe1\xe3\xe4\xf0\xf3\xf4\xf5\xf6\xf7/ ,.:;?!\`^_\/|\'\"()[]{}+\-=<>$%#&*@/; #"'`
            }elsif ($1 eq "\xa3"){
                $ret .= $c, next if $c =~ tr/\xc1-\xda\xe1-\xfa\xb0-\xb9/A-Za-z0-9/;
            }
        }
        $ret .= $tmp;
    }
    return $ret;
}


sub iocode::base64w{
    my $str = shift;

    $str =~ tr/A-Za-z0-9\+\/\=/\x00-\x3f/d;

    my $ord1 = ord(substr($str, 1));
    my $ret = chr((ord($str) << 2 | $ord1 >> 4) & 0xff);

    if (2 < length $str){
        my $ord2 = ord(substr($str, 2));
        $ret .= chr(($ord1 << 4 | $ord2 >> 2) & 0xff);
        if (3 < length $str){
            $ret .= chr(($ord2 << 6 | ord(substr($str, 3))) & 0xff);
        }
    }
    return $ret;
}

sub iocode::base64{
    my $str = shift;
    my($ret);

    $ret .= &iocode::base64w($1) while $str =~ s/^(....)//;
    $ret;
}

sub iocode::quoted{
    my $str = shift;
    my $ret;
    while ($str !~ /^$/){
        $ret .= ' ', next if $str =~ s/^_//;
        $ret .= chr(hex($1)), next if $str =~ s/^=([a-zA-Z0-9]{2})//;
        $ret .= $1 if $str =~ s/^(.)//;
    }
    $ret;
}

sub esc2euc{
    my ($c1, $c2, $str) = @_;
    if ($c1 eq "\$"){
        $str =~ tr/\x21-\x7e/\xa1-\xfe/;
    }elsif ($c2 eq 'I'){
        $str =~ s/(.)/"\x8e" . chr(0x80|ord $1)/ge;
    }
    return $str;
}

sub mime_decode{
    my $ptr = shift;
    my $isref = (ref $ptr);
    my $str = $isref? $$ptr: $ptr;
    my $stat = '';
    if ($str =~ s/=\?ISO-2022-JP\?B\?(([a-zA-Z0-9\+\/\=]{4})+?)\?=/&iocode::base64($1)/ieg){
        $stat = 'ISO-2022-JP';
    }elsif ($str =~ s/=\?ISO-2022-JP\?Q\?((=[0-9a-fA-F][0-9a-fA-F]|[_\x21-\x3c\x3e\x40-\x7e])+)\?=/&iocode::quoted($1)/ieg){
        $stat = 'Shift_JIS';
    }
    $$ptr = $str, return $stat if $isref;
    return $str;
}

sub jis_to_euc{
    my $ptr = shift;
    my $isref = (ref $ptr);
    my $str = $isref? $$ptr: $ptr;
    my $stat = ($str =~ s/\e([\$\(])(.)([\x21-\x7e]*)/&esc2euc($1, $2, $3)/ge);
    $$ptr = $str, return $stat if $isref;
    return $str;
}

sub code_reset{
    $default_input_ja_code = '';
    $last_input_ja_code = '';
    $doc_input_ja_code = '';
}
sub toEuc{
    my $str = shift;
    my %para = ('k2e' => 1);
    my $opt;
    for $opt (@_){
        if (ref $opt eq 'HASH'){
            my $key;
            for $key (%$opt){
                $para{$key} = $opt->{$key};
            }
        }else{
            $para{$opt} = 1;
        }
    }

    my $flag = ($str =~ s/=\?ISO-2022-JP\?B\?(([a-zA-Z0-9\+\/\=]{4})+?)\?=/&iocode::base64($1)/ieg or $str =~ s/=\?ISO-2022-JP\?Q\?((=[0-9a-fA-F][0-9a-fA-F]|[_\x21-\x3c\x3e\x40-\x7e])+)\?=/&iocode::quoted($1)/ieg);
    $flag = 1 if $str =~ s/\e([\$\(])(.)([\x21-\x7e]*)/&esc2euc($1, $2, $3)/ge;
    if ($flag){
        my $euc = $str;
        return $euc if &euc_to_euc(\$euc, \%para) || !&shiftjis_to_euc(\$str, \%para);
        return $str;
    }

    my $sj = $str;
    my $sj_stat = undef;
    my $euc_stat = undef;

    if ($para{'doc'}){
        if ($doc_input_ja_code eq 'Shift_JIS'){
            $sj_stat = &shiftjis_to_euc(\$sj, \%para);
            return $sj if $sj_stat;
            $doc_input_ja_code = '';
        }elsif ($doc_input_ja_code eq 'EUC-JP'){
            $euc_stat = &euc_to_euc(\$str, \%para);
            return $str if $euc_stat;
            $doc_input_ja_code = '';
        }
    }
    $euc_stat = &euc_to_euc(\$str, \%para) unless defined $euc_stat;
    $sj_stat = &shiftjis_to_euc(\$sj, \%para) unless defined $sj_stat;

    if ($para{'k'}){
        $euc_stat |= 0x9;
        $sj_stat |= 0x9;
    }

    if ($sj_stat < $euc_stat){
        $doc_input_ja_code = 'EUC-JP' unless $sj_stat;
        $last_input_ja_code = 'EUC-JP';
        return $str;
    }
    if ($sj_stat > $euc_stat){
        $doc_input_ja_code = 'Shift_JIS' unless $euc_stat;
        $last_input_ja_code = 'Shift_JIS';
        return $sj;
    }
    return $sj if $default_input_ja_code eq 'Shift_JIS';
    return $str if $default_input_ja_code eq 'EUC-JP';

    return $sj if $last_input_ja_code eq 'Shift_JIS';
    return $str if $last_input_ja_code eq 'EUC-JP';
    return $sj if &get_output_ja_code eq 'Shift_JIS';
    return $str if &get_output_ja_code eq 'EUC-JP';
    return $sj if $input_ja_code eq 'Shift_JIS';
    return $str;
}

sub iocode::tosjis{
    my($c1, $c2) = unpack('CC', shift);
    return pack('C', $c2) if $c1 == 0x8e;
    $c2 -= ($c1 & 1)? (0x60 + ($c2 < 0xe0)): 2;
    $c1 = ($c1 + 0x61) >> 1;
    $c1 += 0x40 if $c1 >= 0xa0;
    pack('CC', $c1, $c2);
}

sub euc_to_shiftjis{
    my $ptr = shift;
    my $isref = (ref $ptr);
    my $str = $isref? $$ptr: $ptr;
    my $stat = ($str =~ s/([\x80-\xff].)/&iocode::tosjis($1)/ge);
    $$ptr = $str, return $stat if $isref;
    return $str;
}

sub output_code{
    my $str = shift;
    if ($output_ja_code eq 'ISO-2022-JP'){
        $str =~ s/([\x80-\xff]+)/\e\$B$1\e\(B/g;
        $str =~ tr/\x80-\xff/\x00-\x7f/;
    }elsif ($output_ja_code eq 'Shift_JIS'){
        $str = &euc_to_shiftjis($str);
    }
    return $str;
}

&code_reset;
&lang;
1;
#----------------------- End of 'iocode' Module ----------------------
