#--------------------- 'load namazu.conf' Module ---------------------
sub read_conf{
    my $filename = shift;
    local(*FH);
    return unless open(FH, $filename);
    $NamazuConfFile = $filename;

    &debug_log("RcFile=$filename");

    my $logging;
    while(<FH>){
        next if /^[\# ]/;
        next if /^\s*$/;
        chomp;
        &debug_log("Rc: $_");
        $DEFAULT_DIR      = $2 if /^(INDEX|DEFAULT_DIR)\s+(\S+)/i;
        $TEMPLATE         = $1 if /^Template\s+(\S+)/i;
        $BASE_URL         = $1 if /^BASE_URL\s+(\S+)/i;
        if (/^REPLACE\t+([^\t]+)\t+([^\t]+)/i){
            $ReplaceTo{"\L$1"} = $2;
            push(@ReplaceFrom, $1);
            $URL_REPLACE_FROM = $1;
            $URL_REPLACE_TO = $2;
        }elsif (/^\s*Replace\s+\"((\\.|[^\\])+)\"\s+\"((\\.|[^\\])+)\"/i){ #"
            &entry_replace($1, $3);
        }elsif (/^\s*Replace\s+(\S+)\s+(\S+)/i){
            &entry_replace($1, $2);
        }
        $URL_REPLACE_FROM = $1 if /^URL_REPLACE_FROM\s+(\S+)/i;
        $URL_REPLACE_TO   = $1 if /^URL_REPLACE_TO\s+(\S+)/i;
        $WAKATI           = $2 if /^(wakachi|wakati)\s+(\S+)/i;
        $logging          = $1 if /^LOGGING\s+(\S+)/i;
        $CnfLang          = $2 if /^LANG(UAGE)?\s+(\S+)/i;
        $SCORING          = $1 if /^SCORING\s+(\S+)/i;
        $RespTextOrig     = $1 if /^RespTextOrig\s+(\S+)/i;
        $MakeSummary      = $1 if /^MakeSummary\s+(\S+)/i;
        $MakeGrep         = $1 if /^MakeGrep\s+(\S+)/i;
        $DiaryDir         = $1 if /^DiaryDir\s+(\S+)/i;
        $TinyMknmz        = $1 if /^TinyMknmz\s+(\S+)/i;
        $Debug            = 1  if /^Debug/i;
        $TitleFormat      = $1 if /^TitleFormat\s+(\S.*)$/i;
        $PhoneX0201       = $1 if /^PhoneX0201\s+(\S.*)$/i;
        if (/^EmphasisTags\s+\"(.+)\"\s+\"(.+)\"\s*$/i){ #"
            ($EmTagS, $EmTagE) = ($1, $2);
            $EmTagS =~ s/\\(.)/$1/g;
            $EmTagE =~ s/\\(.)/$1/g;
        }
    }
    close(FH);

    $LOGGING = ($logging =~ /^off/)? 0: 1 if $logging ne '';
    $TFIDF = ($SCORING =~ /tfidf/i);
}

sub load_namazu_conf{
    my($env, $conf, $tmp);

    &in_hns;
    my @conf = ('/usr/local/namazu/conf', '/usr/local/share/namazu/conf',
                '/usr/local/namazu/lib', '/usr/local/etc/namazu',
                '/usr/local/share/etc/namazu', @INC, '.');
    push(@conf, $1), push(@conf, $2) if $0 =~ /^((.*)\/public_html)/;
    push(@conf, $NamazuDir) if $NamazuDir;
    push(@conf, $NamazuConf) if $NamazuConf;
    push(@conf, $ENV{'NAMAZUCONF'}) if $ENV{'NAMAZUCONF'};
    push(@conf, $ENV{'NAMAZUCONFPATH'}) if $ENV{'NAMAZUCONFPATH'};
    push(@conf, $ENV{'HOME'}) if $ENV{'HOME'};
    push(@conf, $ENV{'PWD'}) if $ENV{'PWD'};
    push(@conf, $DiaryConf) if -d $DiaryConf;
    push(@conf, $BaseDir) if $BaseDir;
    push(@conf, $NamazuConfFile) if $NamazuConfFile;

    my (@files) = ("namazu.conf", "namazurc", ".namazurc", ".pnamazurc");

    for $tmp (&pnmz_find(\@conf, \@files)){
        &read_conf($tmp);
    }

    $ReplaceFrom = join('|', map {quotemeta($_)} sort {length $b <=> length $a} @ReplaceFrom);

    if ($Debug > 0){
        if (%ReplaceTo){
            my $key;
            for $key (sort keys %ReplaceTo){
                &debug_log("ReplaceTo: $key => $ReplaceTo{$key}");
            }
        }
        if (@ReplacePattern){
            my $key;
            for $key (@ReplacePattern){
                &debug_log("ReplacePattern: $key => $ReplacePattern{$key}");
            }
        }
        &debug_log("EmStart: $EmTagS");
        &debug_log("EmEnd: $EmTagE");
    }else{
        $Debug = -1;
    }
}

sub entry_replace{
    my ($src, $dst) = @_;
    local($_);
    eval {s/$src/$dst/i};
    unless ($@){
        unshift(@ReplacePattern, $src);
        $ReplacePattern{$src} = $dst;
        $URL_REPLACE_FROM = $src;
        $URL_REPLACE_TO = $dst;
    }
}

sub replace{
    my $src = $_[0];
    return unless $Replace;
    if (%ReplaceTo){
        &debug_log("replace: $src => $_[0]"), return if $_[0] =~ s/($ReplaceFrom)/$ReplaceTo{"\L$1"}/io;
    }
    my $pat;
    for $pat (@ReplacePattern){
        &debug_log("replace: $src => $_[0]"), return if $_[0] =~ s/$pat/$ReplacePattern{$pat}/i;
    }
}

sub in_hns{
    if (-d "./lib/HNS" && -r "config.ph"){
        local(*FH);
        if (open(FH, "config.ph")){
            while (defined(my $str = <FH>)){
                eval("$str") if $str =~ /^\s*\$(DiaryDir)\s*\=/;
            }
            close(FH);
            &debug_log("hns: $DiaryDir");
            if (-d $DiaryDir){
                $DiaryNamazu = "$DiaryDir/namazu2";
                $DiaryNamazu = "$DiaryDir/namazu" unless -d $DiaryNamazu;
                $DiaryConf = "$DiaryNamazu/etc";
                $DiaryTemplate = "$DiaryNamazu/template";
                $DiaryIndex = "$DiaryNamazu/index";
            }
        }
    }
}


sub pnmz_find{
    my ($pres, $files) = @_;
    my ($pre, @ret);
    for $pre (@$pres){
        push(@ret, $pre), next if -f $pre;
        $pre .= '/' if -d $pre and substr($pre, -1) ne '/';
        my $file;
        for $file (@$files){
            my $path = "$dir$file";
            push(@ret, $path) if -r $path;
        }
    }
    @ret;
}

1;
#----------------- End of 'load namazu.conf' Module ------------------
